// MODULE - HIGHSCORES.C
//
// Copyright (c) 2012 Mark Ball. All rights reserved.
// Started by Mark Ball, September 2012
//
// High score handling.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "game.h"
#include "chuckie.h"
#include <e2prom.h>

#pragma dontwarn 214 // VBCC - Suspicious format string.

#define DEBOUNCE_TICKS 5

#define MAX_HIGH_SCORES 10

#define MAX_NAME_WIDTH_IN_BYTES 3

// Title
#define HSC_START_TITLE_Y 2

// Score
#define HSC_SCORE_START_Y (2+18+4)

// League.
#define HSC_LEAGUE_FONT_GAP_Y_IN_PIXELS 16
#define HSC_LEAGUE_START_X (SCREEN_WIDTH_IN_PIXELS/2)
#define HSC_LEAGUE_START_Y (((200-(MAX_HIGH_SCORES*HSC_LEAGUE_FONT_GAP_Y_IN_PIXELS))/2)+(HSC_LEAGUE_FONT_GAP_Y_IN_PIXELS/2))

// High score name entry.
#define HIGH_SCORE_NAME_ENTRY_WIDTH_IN_PIXELS 320
#define HIGH_SCORE_NAME_ENTRY_HEIGHT_IN_PIXELS 200

extern const unsigned char KHighScoreNameEntryBitmapData[];

// High score grid configuration.
#define HSC_GRID_GAP_X      24
#define HSC_GRID_GAP_Y      22
#define HSC_GRID_WIDTH      8
#define HSC_GRID_HEIGHT     5
#define HSC_GRID_START_Y    (40+10)
#define HSC_GRID_START_X    50
#define HSC_GRID_DEL_START_X (HSC_GRID_START_X+(3*HSC_GRID_GAP_X))
#define HSC_GRID_OK_START_X (HSC_GRID_DEL_START_X+HSC_GRID_GAP_X)
#define HSC_GRID_MIN_X HSC_GRID_START_X
#define HSC_GRID_MAX_X (HSC_GRID_START_X+((HSC_GRID_WIDTH-1)*HSC_GRID_GAP_X))
#define MAX_AVAILABLE_NAME_CHARS (HSC_GRID_WIDTH*HSC_GRID_HEIGHT)

// Digit box
#define HSC_DIGIT_BOX_X (320-72-5)
#define HSC_DIGIT_BOX_Y (HSC_GRID_START_Y+12)
#define HSC_DIGIT_BOX_WIDTH_IN_PIXELS   72
#define HSC_DIGIT_BOX_HEIGHT_IN_PIXELS  24

// Curspr
#define HSC_CURSOR_WIDTH_IN_PIXELS  24
#define HSC_CURSOR_HEIGHT_IN_PIXELS 22
#define HSC_CURSOR_X ((HSC_GRID_START_X-((HSC_CURSOR_WIDTH_IN_PIXELS-LARGE_FONT_WIDTH_IN_PIXELS)/2))>>1)
#define HSC_CURSOR_Y (HSC_GRID_START_Y+((HSC_CURSOR_HEIGHT_IN_PIXELS-LARGE_FONT_HEIGHT_IN_PIXELS)/2))
#define HSC_CURSOR_MIN_X (HSC_CURSOR_X<<1)
#define HSC_CURSOR_MAX_X (HSC_CURSOR_MIN_X+((HSC_GRID_WIDTH-1)*HSC_GRID_GAP_X))
#define HSC_CURSOR_MIN_Y HSC_CURSOR_Y
#define HSC_CURSOR_MAX_Y (HSC_CURSOR_Y+((HSC_GRID_HEIGHT-1)*HSC_GRID_GAP_Y))

#define HSC_CURSOR_SPECIAL_MIN_X (HSC_CURSOR_MIN_X+(3*HSC_GRID_GAP_X))
#define HSC_CURSOR_SPECIAL_MAX_X (HSC_CURSOR_MIN_X+(5*HSC_GRID_GAP_X))

// Bitmap indexes.
#define HSC_PARTS_BOX_INDEX 0
#define HSC_PARTS_CURSOR_INDEX (HSC_DIGIT_BOX_WIDTH_IN_PIXELS*HSC_DIGIT_BOX_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define HSC_PARTS_SPINNER_INDEX (HSC_PARTS_CURSOR_INDEX+(HSC_CURSOR_WIDTH_IN_PIXELS*HSC_CURSOR_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES))

extern const unsigned char KHighScoreNameEntryPartsBitmapData[];

// Box digits
#define HSC_BOX_CHAR_OFF_SCREEN_X -50
#define HSC_BOX_CHAR_OFFSET_X 6
#define HSC_BOX_CHAR_OFFSET_Y 10
#define HSC_BOX_CHAR_GAP_X 22
#define HSC_BOX_CHAR_START_X (HSC_DIGIT_BOX_X+HSC_BOX_CHAR_OFFSET_X)
#define HSC_BOX_CHAR_START_Y (HSC_DIGIT_BOX_Y+HSC_BOX_CHAR_OFFSET_Y)
#define HSC_BOX_CHAR_POS1_X (HSC_BOX_CHAR_START_X+(0*HSC_BOX_CHAR_GAP_X))
#define HSC_BOX_CHAR_POS2_X (HSC_BOX_CHAR_START_X+(1*HSC_BOX_CHAR_GAP_X))
#define HSC_BOX_CHAR_POS3_X (HSC_BOX_CHAR_START_X+(2*HSC_BOX_CHAR_GAP_X))

#define HSC_CHAR_IS_MINUS       (MAX_AVAILABLE_NAME_CHARS-4)
#define HSC_CHAR_IS_APERSAND    (MAX_AVAILABLE_NAME_CHARS-3)
#define HSC_CHAR_IS_PLING       (MAX_AVAILABLE_NAME_CHARS-2)
#define HSC_CHAR_IS_QUESTION    (MAX_AVAILABLE_NAME_CHARS-1)
#define HSC_CHAR_IS_DELETE      43
#define HSC_CHAR_IS_OK          44

extern const unsigned char KlargeFontBitmap[];
extern const unsigned char KlargeFontSpaceBitmap[];

static const TSpriteFrameType theNameFontSprite=
    {
    &KlargeFontBitmap[0],LARGE_FONT_WIDTH_IN_PIXELS,LARGE_FONT_HEIGHT_IN_PIXELS
    };

// Objects.
#define HSC_OBJ_CURSOR_INDEX        0
#define HSC_OBJ_NAME_CHAR1_INDEX    1
#define HSC_OBJ_NAME_CHAR2_INDEX    2
#define HSC_OBJ_NAME_CHAR3_INDEX    3
#define HSC_OBJ_SPINNER_INDEX       4

extern TBitmapObj gameObjHighScoreNameEntry[5];

static const TSpriteFrameType theCursorSprite=
    {
    &KHighScoreNameEntryPartsBitmapData[HSC_PARTS_CURSOR_INDEX],HSC_CURSOR_WIDTH_IN_PIXELS,HSC_CURSOR_HEIGHT_IN_PIXELS
    };

// Spinner.
#define HSC_SPINNER_WIDTH_IN_PIXELS     32
#define HSC_SPINNER_HEIGHT_IN_PIXELS    32
#define HSC_SPINNER_ACTION_COUNT        6
#define HSC_SPINNER_ANIMATION_FRAMES    8

static const TSpriteFrameType theSpinnerSprite=
    {
    &KHighScoreNameEntryPartsBitmapData[HSC_PARTS_SPINNER_INDEX],HSC_SPINNER_WIDTH_IN_PIXELS,HSC_SPINNER_HEIGHT_IN_PIXELS
    };

// High score element.
typedef struct
    {
    unsigned char iName[MAX_NAME_WIDTH_IN_BYTES+1];
    unsigned long int iScore;
    } THighScoreEntry;

// Default names and scores.
static THighScoreEntry theLeague[MAX_HIGH_SCORES]=
    {
    {"A&F",1000},
    {"A&F",1000},
    {"A&F",1000},
    {"A&F",1000},
    {"A&F",1000},
    {"A&F",1000},
    {"A&F",1000},
    {"A&F",1000},
    {"A&F",1000},
    {"A&F",1000}
    };

// Positions.
static const char* const KScoreLineStart[]=
    {
    " 1ST",
    " 2ND",
    " 3RD",
    " 4TH",
    " 5TH",
    " 6TH",
    " 7TH",
    " 8TH",
    " 9TH",
    "10TH"
    };

static const char* const KHscChars="ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-&!?";

// Data markers.
#define VALID_DATA_MARKER   'G'
#define VALID_DATA_CHECKSUM 'B'

// Storage indexes.
#define E2PROM_SIZE_IN_BYTES        128
#define VALID_START_MARKER_INDEX    0
#define HSC_LEAGUE_INDEX            1
#define GRAPHICS_STYLE_INDEX        121
#define LEVEL_PACK_INDEX            122
#define MUSIC_VOLUME_INDEX          123
#define FX_VOLUME_INDEX             124
#define GAME_CHECK_SUM_INDEX        125
#define CHECKSUM_INDEX              126 // 2 bytes.

void SaveDataToCart(void)
//
// Save data o cart.
//
    {
    unsigned char buf[E2PROM_SIZE_IN_BYTES];

    // Ensure buffer starts from a known state.
    bzero(&buf[0],E2PROM_SIZE_IN_BYTES);

    buf[VALID_START_MARKER_INDEX]=VALID_DATA_MARKER;

    // Take a copy of the HSC league.
    memcpy(&buf[HSC_LEAGUE_INDEX],(unsigned char*)&theLeague[0].iName[0],sizeof(theLeague));

    // Music/FX volumes.
    buf[MUSIC_VOLUME_INDEX]=gameMusicVolume;
    buf[FX_VOLUME_INDEX]=gameFxVolume;

    // Level pack and graphics style.
    buf[GRAPHICS_STYLE_INDEX]=gameDisplayGraphics;
    buf[LEVEL_PACK_INDEX]=gameDisplayLevelPack;

    // Compute a secondary checksum specific to the game.
    unsigned char checksum=0;
    unsigned char *pD=&buf[0];
    for (short int c=0; c<GAME_CHECK_SUM_INDEX; c++)
        checksum+=*pD++;
    buf[GAME_CHECK_SUM_INDEX]=checksum^VALID_DATA_CHECKSUM;

    // Update the EEPROM.
    E2DataWrite(&buf[0]);
    }

void LoadDataFromCart(void)
//
// Save data o cart.
//
    {
    unsigned char buf[E2PROM_SIZE_IN_BYTES];

    // Ensure buffer starts from a known state.
    bzero(&buf[0],E2PROM_SIZE_IN_BYTES);

    // Read the data
    if (E2DataRead(&buf[0])==0) // Checksum OK
        if (buf[VALID_START_MARKER_INDEX]==VALID_DATA_MARKER)   // Start marker valid?
            {
            // Check the secondary checksum.
            unsigned char checksum=0;
            unsigned char *pD=&buf[0];

            for (short int c=0; c<GAME_CHECK_SUM_INDEX; c++)
                checksum+=*pD++;

            // Secondary checksum good?
            if ((buf[GAME_CHECK_SUM_INDEX]^checksum)==VALID_DATA_CHECKSUM)
                {
                // Restore HSC.
                memcpy(&theLeague[0].iName[0],&buf[HSC_LEAGUE_INDEX],sizeof(theLeague));

                // Restore music/FX volumes.
                gameMusicVolume=buf[MUSIC_VOLUME_INDEX];
                gameFxVolume=buf[FX_VOLUME_INDEX];

                // Restore level pack and graphics style.
                gameDisplayGraphics=buf[GRAPHICS_STYLE_INDEX];
                gameDisplayLevelPack=buf[LEVEL_PACK_INDEX];
                }
            }
    }

void DrawHighScores(void)
//
// Draw the high score league.
//
    {
    char buf[41];
    char paddedName[MAX_NAME_WIDTH_IN_BYTES+1]={0,0,0,0};

    gameTextIsCentred=TRUE;
    SetFontSize(FONT_USED_IS_LARGE);
    DrawFont("HIGH SCORES",HSC_LEAGUE_START_X,2);

    for (unsigned short int c=0; c<MAX_HIGH_SCORES; c++)
        {
        unsigned char* pSrc=&theLeague[c].iName[0];
        unsigned char* pDest=&paddedName[0];

        // Replace NULLs with spaces.
        for (short int p=0; p<MAX_NAME_WIDTH_IN_BYTES; p++)
            {
            const unsigned char d=*pSrc++;
            *pDest++=(d!='\0')?d:' ';
            }

        sprintf(&buf[0],"%s %s %07d",KScoreLineStart[c],&paddedName[0],theLeague[c].iScore);
        DrawFont(&buf[0],HSC_LEAGUE_START_X,HSC_LEAGUE_START_Y+(c*HSC_LEAGUE_FONT_GAP_Y_IN_PIXELS));
        }
    }

static const short int KHscObjectOffsets[3]=
    {
    HSC_BOX_CHAR_POS1_X,
    HSC_BOX_CHAR_POS2_X,
    HSC_BOX_CHAR_POS3_X
    };

static const unsigned long KMultiplyTable[8]=
    {
    10000000L,
    1000000L,
    100000L,
    10000L,
    1000L,
    100L,
    10L,
    1L,
    };

static unsigned char theNameLockedIn;
static short int theSpinnerActionCount;
static short int theSpinnerFrame;
static const unsigned char *theSpinnerDataPtr;

static void SpinnerAnimate(void)
//
// Animate the spinner.
//
    {

    WaitForVBlank();

    if (theNameLockedIn)
        if (theSpinnerActionCount++==HSC_SPINNER_ACTION_COUNT)
            {
            theSpinnerActionCount=0;
            theSpinnerDataPtr+=HSC_SPINNER_WIDTH_IN_PIXELS*HSC_SPINNER_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES;

            if (theSpinnerFrame++==HSC_SPINNER_ANIMATION_FRAMES-1)
                {
                theSpinnerFrame=0;
                theSpinnerDataPtr=&KHighScoreNameEntryPartsBitmapData[HSC_PARTS_SPINNER_INDEX];
                }

            gameObjHighScoreNameEntry[HSC_OBJ_SPINNER_INDEX].iPhrase1.iData=((unsigned long int)theSpinnerDataPtr)>>3;
            }
    }

static void SpinnerOffScreen(void)
//
// Move the spinner off screen
//
    {

    gameObjHighScoreNameEntry[HSC_OBJ_SPINNER_INDEX].iPhrase2.iXPos=HSC_BOX_CHAR_OFF_SCREEN_X+gameObjListOffsetX;
    }

static void SpinnerOnScreen(void)
//
// Move the spinner on screen
//
    {

    gameObjHighScoreNameEntry[HSC_OBJ_SPINNER_INDEX].iPhrase2.iXPos=gameObjListOffsetX+HSC_GRID_OK_START_X-8;
    }

unsigned char HandleNewHighScore(void)
//
// Handle a new high score.
//
    {
    TBitmapBlitType hscNameEntryBmp=
        {
        (unsigned char*)&KHighScoreNameEntryBitmapData[0],
        NULL,
        WID320,
        LFU_REPLACE,
        HIGH_SCORE_NAME_ENTRY_WIDTH_IN_PIXELS,HIGH_SCORE_NAME_ENTRY_HEIGHT_IN_PIXELS,
        (SCREEN_WIDTH_IN_PIXELS-HIGH_SCORE_NAME_ENTRY_WIDTH_IN_PIXELS)/2,
        SCREEN_CROP_HEIGHT_IN_PIXELS+((SCREEN_HEIGHT_VISIBLE_IN_PIXELS-HIGH_SCORE_NAME_ENTRY_HEIGHT_IN_PIXELS)/2)
        };
    TBitmapBlitType hscNameEntryBoxBmp=
        {
        (unsigned char*)&KHighScoreNameEntryPartsBitmapData[HSC_PARTS_BOX_INDEX],
        NULL,
        WID320,
        LFU_REPLACE|DCOMPEN,
        HSC_DIGIT_BOX_WIDTH_IN_PIXELS,HSC_DIGIT_BOX_HEIGHT_IN_PIXELS,
        HSC_DIGIT_BOX_X,
        SCREEN_CROP_HEIGHT_IN_PIXELS+HSC_DIGIT_BOX_Y+6
        };
    unsigned long hsc=0;
    short int c=0;
    unsigned char* pD=&all_player_data[0].score[0];
    char buf[10];

    // Convert the CE score into something easily comparable.
    for ( ; c<8; c++)
        hsc+=*pD++*KMultiplyTable[c];

    // Better than any of the others?
    unsigned char *pName=NULL;
    THighScoreEntry* pEntry=&theLeague[0];
    for (c=0; c<MAX_HIGH_SCORES; c++)
        {
        if (hsc>=pEntry->iScore)
            {
            // Insert the new HSC.
            memmove(pEntry+1,pEntry,((MAX_HIGH_SCORES-1)-c)*sizeof(THighScoreEntry));
            pEntry->iScore=hsc;
            pName=&pEntry->iName[0];
            break;
            }
        pEntry++;
        }

    // New high score?
    if (pName==NULL)
        return(FALSE);

    // Clear out the old entry.
    pName[0]='\0';
    pName[1]='\0';
    pName[2]='\0';
    pName[4]='\0';

    // Clear screen and blit the background image.
    hscNameEntryBmp.iDest=gameBackScreen;
    BlitClearScreen();
    BlitBitmap(&hscNameEntryBmp);

    hscNameEntryBoxBmp.iDest=gameBackScreen;
    BlitBitmap(&hscNameEntryBoxBmp);

    // Title and chars next.
    gameTextIsCentred=TRUE;
    SetFontSize(FONT_USED_IS_LARGE);
    DrawFont("NEW HIGH SCORE",160,HSC_START_TITLE_Y);
    sprintf(&buf[0],"%07d",hsc);
    DrawFont(&buf[0],160,HSC_SCORE_START_Y);

    gameTextIsCentred=FALSE;
    const char* pHscData=&KHscChars[0];
    short int yPos=HSC_GRID_START_Y;
    buf[1]='\0';
    for (short int y=0; y<5; y++)
        {
        short int xPos=HSC_GRID_START_X;
        for (short int x=0; x<HSC_GRID_WIDTH; x++)
            {
            buf[0]=*pHscData++;
            DrawFont(&buf[0],xPos,yPos);
            xPos+=HSC_GRID_GAP_X;
            }
        yPos+=HSC_GRID_GAP_Y;
        }

    DrawFont("(",HSC_GRID_OK_START_X,yPos);
    DrawFont(")",HSC_GRID_DEL_START_X,yPos);

    FlipScreens();
    WaitForVBlank();

    // Initialise the cursor object.
    MakeBitmapObj(&gameObjHighScoreNameEntry[HSC_OBJ_CURSOR_INDEX],&theCursorSprite,HSC_CURSOR_X,HSC_CURSOR_Y);
    gameObjHighScoreNameEntry[HSC_OBJ_CURSOR_INDEX].iPhrase1.iYPos=gameObjListOffsetY+(HSC_CURSOR_Y<<1);

    // Initialise the first name objects.
    MakeBitmapObj(&gameObjHighScoreNameEntry[HSC_OBJ_NAME_CHAR1_INDEX],&theNameFontSprite,HSC_BOX_CHAR_POS1_X>>1,HSC_BOX_CHAR_START_Y);
    gameObjHighScoreNameEntry[HSC_OBJ_NAME_CHAR1_INDEX].iPhrase1.iYPos=gameObjListOffsetY+(HSC_BOX_CHAR_START_Y<<1);

    // The next two name objects are off screen (to start with).
    MakeBitmapObj(&gameObjHighScoreNameEntry[HSC_OBJ_NAME_CHAR2_INDEX],&theNameFontSprite,HSC_BOX_CHAR_OFF_SCREEN_X>>1,HSC_BOX_CHAR_START_Y);
    gameObjHighScoreNameEntry[HSC_OBJ_NAME_CHAR2_INDEX].iPhrase1.iYPos=gameObjListOffsetY+(HSC_BOX_CHAR_START_Y<<1);
    MakeBitmapObj(&gameObjHighScoreNameEntry[HSC_OBJ_NAME_CHAR3_INDEX],&theNameFontSprite,HSC_BOX_CHAR_OFF_SCREEN_X>>1,HSC_BOX_CHAR_START_Y);
    gameObjHighScoreNameEntry[HSC_OBJ_NAME_CHAR3_INDEX].iPhrase1.iYPos=gameObjListOffsetY+(HSC_BOX_CHAR_START_Y<<1);

    // Initialise the spinner object.
    MakeBitmapObj(&gameObjHighScoreNameEntry[HSC_OBJ_SPINNER_INDEX],&theSpinnerSprite,HSC_BOX_CHAR_OFF_SCREEN_X>>1,0);
    gameObjHighScoreNameEntry[HSC_OBJ_SPINNER_INDEX].iPhrase1.iYPos=gameObjListOffsetY+((yPos-2)<<1);

    short int cursorX=HSC_CURSOR_MIN_X;
    short int cursorMinX=HSC_CURSOR_MIN_X;
    short int cursorMaxX=HSC_CURSOR_MAX_X;
    short int cursorY=HSC_CURSOR_MIN_Y;
    short int cursorMaxY=HSC_CURSOR_MAX_Y;
    short int lastNameIndex=0;
    TBitmapObj* pNameObject=&gameObjHighScoreNameEntry[HSC_OBJ_NAME_CHAR1_INDEX];
    short int reel=0;

    theNameLockedIn=FALSE;
    theSpinnerActionCount=0;
    theSpinnerFrame=0;
    theSpinnerDataPtr=&KHighScoreNameEntryPartsBitmapData[HSC_PARTS_SPINNER_INDEX];

    for (;;)
        {
        unsigned long int what;
        for (;;)
            {
            SpinnerAnimate();
            if ((what=JoyPad())&(JOYPAD_MOVE|JOYPAD_FIREB))
                break;
            }

        // Handle the pad input.
        if (what&JOYPAD_RIGHT)
            cursorX+=HSC_GRID_GAP_X;
        else if (what&JOYPAD_LEFT)
            cursorX-=HSC_GRID_GAP_X;
        if (what&JOYPAD_UP)
            cursorY-=HSC_GRID_GAP_Y;
        else if (what&JOYPAD_DOWN)
            cursorY+=HSC_GRID_GAP_Y;

        // Change the end stops?
        if ((cursorX>=HSC_CURSOR_SPECIAL_MIN_X)&&(cursorX<HSC_CURSOR_SPECIAL_MAX_X))
            cursorMaxY=HSC_CURSOR_MAX_Y+HSC_GRID_GAP_Y;
        else
            cursorMaxY=HSC_CURSOR_MAX_Y;

        // X cursor limits.
        if (cursorX<cursorMinX)
            cursorX=cursorMaxX;
        else if (cursorX>cursorMaxX)
            cursorX=cursorMinX;

        // Y cursor limits.
        if (cursorY<HSC_CURSOR_MIN_Y)
            cursorY=cursorMaxY;
        else if (cursorY>cursorMaxY)
            cursorY=HSC_CURSOR_MIN_Y;

        // Update the cursor object.
        gameObjHighScoreNameEntry[HSC_OBJ_CURSOR_INDEX].iPhrase2.iXPos=gameObjListOffsetX+cursorX;
        gameObjHighScoreNameEntry[HSC_OBJ_CURSOR_INDEX].iPhrase1.iYPos=gameObjListOffsetY+(cursorY<<1);

        // Compute the index we need.
        const short int newIndex=(((cursorY-HSC_CURSOR_MIN_Y)/HSC_GRID_GAP_Y)*HSC_GRID_WIDTH)+
                                ((cursorX-HSC_CURSOR_MIN_X)/HSC_GRID_GAP_X);

        // Is it a displayable character?
        if (newIndex<MAX_AVAILABLE_NAME_CHARS)
            {
            // Chec the special ones first.
            if (newIndex==HSC_CHAR_IS_MINUS)
                lastNameIndex=FONT_MINUS_INDEX;
            else if (newIndex==HSC_CHAR_IS_APERSAND)
                lastNameIndex=FONT_AMPERSAND_INDEX;
            else if (newIndex==HSC_CHAR_IS_PLING)
                lastNameIndex=FONT_PLING_INDEX;
            else if (newIndex==HSC_CHAR_IS_QUESTION)
                lastNameIndex=FONT_QUESTION_MARK_INDEX;
            else
                lastNameIndex=newIndex;

            if (!theNameLockedIn)
                pNameObject->iPhrase1.iData=((unsigned long int)&KlargeFontBitmap[lastNameIndex*LARGE_FONT_SIZE_IN_BYTES])>>3;
            }
        else if (!theNameLockedIn)
            pNameObject->iPhrase1.iData=((unsigned long int)&KlargeFontSpaceBitmap[0])>>3;

        // If the name is locked in and the cursor is over the OK font
        // entry then move the spinner off screen.
        if (theNameLockedIn)
            {
            if (newIndex==HSC_CHAR_IS_OK)
                SpinnerOffScreen();
            else
                SpinnerOnScreen();
            }

        SooundEffectNext();

        if (what&JOYPAD_FIREB)
            {
            if (newIndex==HSC_CHAR_IS_DELETE)
                {
                SooundEffectCancel();
                if (theNameLockedIn)
                    {
                    theNameLockedIn=FALSE;
                    SpinnerOffScreen();
                    }
                else
                    {
                    // Clear last selection.
                    pName[reel]='\0';

                    // Move to the previous reel.
                    if (reel>=1)
                        {
                        pNameObject->iPhrase2.iXPos=HSC_BOX_CHAR_OFF_SCREEN_X+gameObjListOffsetX;
                        pNameObject--;
                        reel--;
                        }
                    }

                // Blank it.
                pNameObject->iPhrase1.iData=((unsigned long int)&KlargeFontSpaceBitmap[0])>>3;
                }
            else if (newIndex==HSC_CHAR_IS_OK)
                break;
            else if (!theNameLockedIn)
                {
                SooundEffectAccept();

                pName[reel]=KHscChars[newIndex];
                if (reel<2)
                    {
                    pNameObject++;
                    reel++;
                    }
                else
                    {
                    theNameLockedIn=TRUE;
                    SpinnerOnScreen();
                    }

                pNameObject->iPhrase1.iData=((unsigned long int)&KlargeFontBitmap[lastNameIndex*LARGE_FONT_SIZE_IN_BYTES])>>3;
                pNameObject->iPhrase2.iXPos=KHscObjectOffsets[reel]+gameObjListOffsetX;
                }

            // Wait for fire to come up.
            for (;;)
                {
                SpinnerAnimate();
                if ((JoyPad()&JOYPAD_FIREB)==0)
                    break;
                }
            }

        // Wait for a bit to debounce
        gameSeconds=0;
        gameTicks=0;
        for (;;)
            {
            SpinnerAnimate();
            if (gameTicks==DEBOUNCE_TICKS)
                break;
            }
        }

    // Commit the data to cart.
    SaveDataToCart();

    // Go straight to league table in the title sequence.
    gameDisplayHighScoreLeague=TRUE;

    return(TRUE);
    }

#pragma popwarn // Suspicious format string.

