// MODULE - OPTIONS.C
//
// Copyright (c) 2012 Mark Ball. All rights reserved.
// Started by Mark Ball, September 2012
//
// Options screen handler.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "game.h"

// Define the following to enable the ability to start on any level.
// #define __ALLOW_START_ON_ANY_LEVEL__

#if !defined(__ALLOW_START_ON_ANY_LEVEL__)
#define MAX_OPTIONS 5
#else
#define MAX_OPTIONS 6
#endif
#define MENU_OPTION_IS_GRAPHICS     0
#define MENU_OPTION_IS_LEVEL_PACK   1
#define MENU_OPTION_IS_MUSIC_VOLUME 2
#define MENU_OPTION_IS_FX_VOLUME    3
#if defined(__ALLOW_START_ON_ANY_LEVEL__)
#define MENU_OPTION_IS_START_LEVEL  4
#endif
#define MENU_OPTION_IS_EXIT         (MAX_OPTIONS-1)

#define GUI_MUSIC_VOLUME_LIMIT_LO   0
#define GUI_MUSIC_VOLUME_LIMIT_HI   (64/VOLUME_MULTIPLIER)
#define GUI_FX_VOLUME_LIMIT_LO      0
#define GUI_FX_VOLUME_LIMIT_HI      (64/VOLUME_MULTIPLIER)
#define GUI_LEVEL_LIMIT_LO          1
#define GUI_LEVEL_LIMIT_HI          40

#define SELECTION_BAR_WIDTH_IN_PIXELS 160
#define SELECTION_BAR_HEIGHT_IN_PIXELS 20
#define SELECTION_TITLE_OFFSET (((3*SELECTION_BAR_HEIGHT_IN_PIXELS)/2)-2)
#define SELECTION_START_Y ((200-(MAX_OPTIONS*SELECTION_BAR_HEIGHT_IN_PIXELS))/2)

#define ACTIVE_INFO_Y (200-SMALL_FONT_HEIGHT_IN_PIXELS-2)

#define STATE_ANIMATION_OPENING     0   // Opening.
#define STATE_ANIMATION_CLOSING     1   // Closing,
#define STATE_ANIMATION_ACTIVE      2   // Active.
#define STATE_ANIMATION_COMPLETE    3   // Done!

#define ANIMATION_STEP_IN_PIXELS 2
#define ANIMATION_COUNT (SELECTION_BAR_HEIGHT_IN_PIXELS/ANIMATION_STEP_IN_PIXELS)

extern unsigned char* KOptionsSelectMaskBitmap[];

static const char* const optionsText[MAX_OPTIONS]=
    {
    "STYLE",
    "LEVEL",
    "MUSIC",
    "FX",
#if defined(__ALLOW_START_ON_ANY_LEVEL__)
    "START ON",
#endif
    "EXIT"
    };


static unsigned char thePlayerChangedValues;
static unsigned char theDisplayGraphicsTemp;
static unsigned char theDisplayLevelPackTemp;
static unsigned char theMusicVolumeTemp;
static unsigned char theFxVolumeTemp;
#if defined(__ALLOW_START_ON_ANY_LEVEL__)
static unsigned char thePlayerStartLevelTemp;
#endif

static void DisplayNumber(const short int aYPos, const short int aNumber)
//
// Display a 2 digit number.
//
    {
    char buf[3]={0,0,0};

    sprintf(&buf[0],"%d",aNumber);
    DrawFont(&buf[0],160,aYPos);
    }

static void ChangeNumber(const unsigned long aDirection,
                        signed char* aNumberPtr,
                        signed char aLimitLo,
                        signed char aLimitHi)
//
// Change a number.
//
    {
    signed char d=*aNumberPtr;

    if (aDirection&JOYPAD_LEFT)
        d--;
    else
        d++;
    if (d<aLimitLo)
        d=aLimitHi;
    else if (d>aLimitHi)
        d=aLimitLo;

    *aNumberPtr=d;
    }

static void DisplayLook(const short int aYPos)
//
// Display the look.
//
    {

    DrawFont(theDisplayGraphicsTemp==LOOK_IS_RETRO?"RETRO":"MODERN",160,aYPos);
    }

static void ChangeDisplayLook(const unsigned long aDirection)
//
// Change the look.
//
    {

    if (aDirection&JOYPAD_LEFT)
        theDisplayGraphicsTemp--;
    else
        theDisplayGraphicsTemp++;

    theDisplayGraphicsTemp&=1;
    }

static void DisplayLevelPack(const short int aYPos)
//
// Display the level pack.
//
    {

    DrawFont(theDisplayLevelPackTemp==LEVEL_PACK_IS_NORMAL?"NORMAL":"BIT TWIDDLERS",160,aYPos);
    }

static void ChangeLevelPack(const unsigned long aDirection)
//
// Change the level pack.
//
    {

    if (aDirection&JOYPAD_LEFT)
        theDisplayLevelPackTemp--;
    else
        theDisplayLevelPackTemp++;

    theDisplayLevelPackTemp&=1;
    }

static void HandleDrawActiveDisplay(short int aChoice,short int aPosY)
//
// Handle the active display draw.
//
    {

    switch (aChoice)
        {
    case MENU_OPTION_IS_GRAPHICS: DisplayLook(aPosY); break;
    case MENU_OPTION_IS_LEVEL_PACK: DisplayLevelPack(aPosY); break;
    case MENU_OPTION_IS_MUSIC_VOLUME: DisplayNumber(aPosY,theMusicVolumeTemp); break;
    case MENU_OPTION_IS_FX_VOLUME: DisplayNumber(aPosY,theFxVolumeTemp); break;
#if defined(__ALLOW_START_ON_ANY_LEVEL__)
    case MENU_OPTION_IS_START_LEVEL: DisplayNumber(aPosY,thePlayerStartLevelTemp); break;
#endif
        }

    // Info at the bottom of the screen.
    SetFontSize(FONT_USED_IS_SMALL);
    DrawFont("B TO SELECT, C TO CANCEL",160,ACTIVE_INFO_Y);
    SetFontSize(FONT_USED_IS_LARGE);
    }

extern void SoundEffectWalking(void);
static void HandleChangeActiveDisplay(const unsigned long int aDirection,const short int aChoice)
//
// Handle a change of active display.
//
    {

    switch (aChoice)
        {
    case MENU_OPTION_IS_GRAPHICS: ChangeDisplayLook(aDirection); break;
    case MENU_OPTION_IS_LEVEL_PACK: ChangeLevelPack(aDirection); break;
    case MENU_OPTION_IS_MUSIC_VOLUME:
        ChangeNumber(aDirection,&theMusicVolumeTemp,GUI_MUSIC_VOLUME_LIMIT_LO,GUI_MUSIC_VOLUME_LIMIT_HI);
        SoundVolumeMusic(theMusicVolumeTemp*4);
    break;
    case MENU_OPTION_IS_FX_VOLUME:
        ChangeNumber(aDirection,&theFxVolumeTemp,GUI_FX_VOLUME_LIMIT_LO,GUI_FX_VOLUME_LIMIT_HI);
        SoundVolumeFx(theFxVolumeTemp*4);
        SoundEffectWalking();
        SoundFxQueueDispatch();
    break;
#if defined(__ALLOW_START_ON_ANY_LEVEL__)
    case MENU_OPTION_IS_START_LEVEL:
        ChangeNumber(aDirection,&thePlayerStartLevelTemp,GUI_LEVEL_LIMIT_LO,GUI_LEVEL_LIMIT_HI);
    break;
#endif
        }

    if (aChoice!=MENU_OPTION_IS_FX_VOLUME)
        SooundEffectNext();
    }

static void HandleAcceptActiveDisplay(const short int aChoice)
//
// Handle active display accept.
//
    {

    switch (aChoice)
        {
    case MENU_OPTION_IS_GRAPHICS: gameDisplayGraphics=theDisplayGraphicsTemp; break;
    case MENU_OPTION_IS_LEVEL_PACK: gameDisplayLevelPack=theDisplayLevelPackTemp; break;
    case MENU_OPTION_IS_MUSIC_VOLUME: gameMusicVolume=theMusicVolumeTemp*VOLUME_MULTIPLIER; break;
    case MENU_OPTION_IS_FX_VOLUME: gameFxVolume=theFxVolumeTemp*VOLUME_MULTIPLIER; break;
#if defined(__ALLOW_START_ON_ANY_LEVEL__)
    case MENU_OPTION_IS_START_LEVEL:
        gamePlayerHasSelectedStartLevel=TRUE;
        gamePlayerStartLevel=thePlayerStartLevelTemp;
    break;
#endif
        }

    // The player changed something!
    thePlayerChangedValues=TRUE;
    }

static void HandleCancelActiveDisplay(const short int aChoice)
//
// Handle active display cancel.
//
    {

    switch (aChoice)
        {
    case MENU_OPTION_IS_GRAPHICS: theDisplayGraphicsTemp=gameDisplayGraphics; break;
    case MENU_OPTION_IS_LEVEL_PACK: theDisplayLevelPackTemp=gameDisplayLevelPack; break;
    case MENU_OPTION_IS_MUSIC_VOLUME:
        theMusicVolumeTemp=gameMusicVolume/VOLUME_MULTIPLIER;
        SoundVolumeMusic(gameMusicVolume);
    break;
    case MENU_OPTION_IS_FX_VOLUME:
        theFxVolumeTemp=gameFxVolume/VOLUME_MULTIPLIER;
        SoundVolumeFx(gameFxVolume);
    break;
#if defined(__ALLOW_START_ON_ANY_LEVEL__)
    case MENU_OPTION_IS_START_LEVEL:
        gamePlayerHasSelectedStartLevel=FALSE;
        thePlayerStartLevelTemp=gamePlayerStartLevel;
    break;
#endif
        }

    SooundEffectCancel();
    }

void HandleOptions(TBitmapBlitType* aBackgroundBmp)
//
// Handle the options menu.
//
    {
    short int selectionBarY;
    short int choice=0;
    short int animationState=STATE_ANIMATION_COMPLETE;
    short int animationCounter=0;
    short int animationY=0;

    // Take copies.
    theDisplayGraphicsTemp=gameDisplayGraphics;
    theDisplayLevelPackTemp=gameDisplayLevelPack;
    theMusicVolumeTemp=gameMusicVolume/VOLUME_MULTIPLIER;
    theFxVolumeTemp=gameFxVolume/VOLUME_MULTIPLIER;
#if defined(__ALLOW_START_ON_ANY_LEVEL__)
    thePlayerStartLevelTemp=gamePlayerStartLevel;
#endif

    // Nothing changed yet,
    thePlayerChangedValues=FALSE;

    // Clear the front and back buffers.
    BlitClearScreen();
    FlipScreens();
    WaitForVBlank();
    BlitClearScreen();
    FlipScreens();

    gameTextIsCentred=TRUE;

    for (;;)
        {
        // Blit the background.
        aBackgroundBmp->iDest=gameBackScreen;
        BlitBitmap(aBackgroundBmp);

        SetFontSize(FONT_USED_IS_LARGE);
        DrawFont("OPTIONS",160,SELECTION_START_Y-SELECTION_TITLE_OFFSET);

        // Display the choices.
        selectionBarY=SCREEN_CROP_HEIGHT_IN_PIXELS+6+SELECTION_START_Y;
        short int choiceY=SELECTION_START_Y+5;
        unsigned char oneShot=FALSE;
        for (short int c=0; c<MAX_OPTIONS; c++)
            {
            // Is it selected?
            if (choice==c)
                {
                AlphaOptionsBar(gameBackScreen+
                                (selectionBarY*SCREEN_WIDTH_IN_BYTES)+
                                (SCREEN_WIDTH_IN_PIXELS-SELECTION_BAR_WIDTH_IN_PIXELS));

                // What is happening to the rest of the menu?
                switch (animationState)
                    {
                case STATE_ANIMATION_OPENING:
                    if (animationCounter--==0)
                        {
                        animationState=STATE_ANIMATION_ACTIVE;

                        // Display the active option.
                        const short int where=choiceY+SELECTION_BAR_HEIGHT_IN_PIXELS;
                        HandleDrawActiveDisplay(choice,where);
                        }
                    else
                        animationY+=ANIMATION_STEP_IN_PIXELS;
                break;
                case STATE_ANIMATION_CLOSING:
                    if (animationCounter--==0)
                        animationState=STATE_ANIMATION_COMPLETE;
                    else
                        animationY-=ANIMATION_STEP_IN_PIXELS;
                break;
                case STATE_ANIMATION_ACTIVE:
                    animationY=SELECTION_BAR_HEIGHT_IN_PIXELS;

                    // Display the active option.
                    const short int where=choiceY+SELECTION_BAR_HEIGHT_IN_PIXELS;
                    HandleDrawActiveDisplay(choice,where);
                break;
                    }

                oneShot=TRUE;
                }

            // Text.
            DrawFont(optionsText[c],160,choiceY);

            if (oneShot&(animationState!=STATE_ANIMATION_COMPLETE))
                {
                oneShot=FALSE;
                selectionBarY+=animationY;
                choiceY+=animationY;
                }

            // Next position.
            selectionBarY+=SELECTION_BAR_HEIGHT_IN_PIXELS;
            choiceY+=SELECTION_BAR_HEIGHT_IN_PIXELS;
            }


        // Flip the screens.
        FlipScreens();
        WaitForVBlank();

        if (animationState>=STATE_ANIMATION_ACTIVE)
            {
            // See what the player wants to do.
            unsigned long int what=0;
            do
                {
                what=JoyPad()&(JOYPAD_MOVE|JOYPAD_FIRE);
                WaitForVBlank();
                } while (what==0);

            // Up/down only allowed when a menu item is not active.
            if (animationState==STATE_ANIMATION_COMPLETE)
                {
                if (what&JOYPAD_UP)
                    {
                    SooundEffectNext();
                    if (--choice<0)
                        choice+=MAX_OPTIONS;
                    }

                if (what&JOYPAD_DOWN)
                    {
                    SooundEffectNext();
                    if (++choice>=MAX_OPTIONS)
                        choice=0;
                    }
                }
            else if (what&(JOYPAD_LEFT|JOYPAD_RIGHT))
                HandleChangeActiveDisplay(what&(JOYPAD_LEFT|JOYPAD_RIGHT),choice);

            // Accepted!
            if (what&JOYPAD_FIREB)
                {
                SooundEffectAccept();

                // Check for exit.
                if (choice==MENU_OPTION_IS_EXIT)
                    break;

                if (animationState==STATE_ANIMATION_COMPLETE)
                    {
                    animationY=0;
                    animationState=STATE_ANIMATION_OPENING;
                    }
                else if (animationState==STATE_ANIMATION_ACTIVE)
                    {
                    animationState=STATE_ANIMATION_CLOSING;
                    HandleAcceptActiveDisplay(choice);
                    }

                animationCounter=ANIMATION_COUNT;
                }
            else if (what&JOYPAD_FIREC) // Cancelled!
                    if (animationState==STATE_ANIMATION_ACTIVE)
                        {
                        animationState=STATE_ANIMATION_CLOSING;
                        animationCounter=ANIMATION_COUNT;
                        HandleCancelActiveDisplay(choice);
                        }

            // Wait for a bit to debounce
            DebounceAnyKey();
            }
        }

    // Update the E2PROM if the player changed something.
    if (thePlayerChangedValues)
        SaveDataToCart();

    // Wait for fire to come up.
    for (;;)
        {
        if ((JoyPad()&JOYPAD_FIRE)==0L)
            break;
        WaitForVBlank();
        }
    }
